@@ -14,7 +14,12 @@
  /* BSDi defines u_intXX_t types in machine/types.h */
 #include <machine/types.h>
 #endif
-#include <dirent.h>
+#ifdef WIN32
+#  include <windows.h>
+#  include <winbase.h>
+#else
+#  include <dirent.h>
+#endif
 
 #include "iostream.hpp"
 
@@ -227,13 +232,14 @@ namespace acommon {
 	to_add->c_struct.dict_dirs = &(to_add->dict_exts);
 	itemize(data, to_add->dict_exts);
       } else {
+fprintf(stderr,"File: %s(%i)\n",__FILE__,__LINE__);
 	err.prim_err(unknown_key, key);
 	goto RETURN_ERROR;
       }
     }
   
     while (*prev != 0 && 
-	   (*prev)->c_struct.order_num > to_add->c_struct.order_num)
+	   (*prev)->c_struct.order_num < to_add->c_struct.order_num)
       prev = &(*prev)->next;
     to_add->next = *prev;
     *prev = to_add;
