@@ -147,7 +147,7 @@ namespace aspeller {
   
     SoundslikeWord() : soundslike(0) {}
     SoundslikeWord(const char * w, const void * p) 
-      : soundslike(w), word_list_pointer(p) {}
+      : soundslike(w == 0 || w[0] == 0 ? 0 : w), word_list_pointer(p) {}
   };
 
   static const unsigned int MaxCompoundLength = 8;
@@ -197,6 +197,13 @@ namespace aspeller {
     void set(const Language * l, const Config * c, bool strip = false);
   };
 
+  class SoundslikeEnumeration 
+  {
+  public:
+    virtual SoundslikeWord next(int) = 0;
+    virtual ~SoundslikeEnumeration() {}
+  };
+
   class BasicWordSet : public LoadableDataSet, public WordList
   {
   public:
@@ -209,9 +216,7 @@ namespace aspeller {
     typedef const char *                  Value;
     typedef unsigned int                  Size;
     typedef SoundslikeWord                SoundslikeValue;
-    typedef VirEnumeration<SoundslikeWord>  VirSoundslikeEmul;
-    typedef Enumeration<VirSoundslikeEmul>  SoundslikeEmul;
-
+    typedef SoundslikeEnumeration         VirSoundslikeEmul;
     StringEnumeration * elements() const;
     virtual VirEmul * detailed_elements() const = 0;
     virtual Size   size()     const = 0;
@@ -269,8 +274,7 @@ namespace aspeller {
     typedef const char *                  Value;
     typedef unsigned int                  Size;
     typedef SoundslikeWord                SoundslikeValue;
-    typedef VirEnumeration<SoundslikeWord>  VirSoundslikeEmul;
-    typedef Enumeration<VirSoundslikeEmul>  SoundslikeEmul;
+    typedef SoundslikeEnumeration  VirSoundslikeEmul;
 
     virtual VirEmul * elements() const = 0;
     virtual Size   size()     const = 0;
@@ -315,7 +319,7 @@ namespace aspeller {
 
     virtual bool   empty()    const {return !size();}
     virtual Size   size()     const = 0;
-    virtual StringEnumeration * elements() const {abort(); return 0; } //FIXME
+    virtual StringEnumeration * elements() const {abort();} //FIXME
 
     virtual VirEmul * detailed_elements() const = 0;
   };
